@@ -712,6 +712,68 @@ void cvSetModeWindow_COCOA( const char* name, double prop_value )
     __END__;
 }
 
+double cvGetPropTopmost_COCOA(const char* name)
+{
+    double    result = -1;
+    CVWindow* window = nil;
+
+    CV_FUNCNAME("cvGetPropTopmost_COCOA");
+
+    __BEGIN__;
+    if (name == NULL)
+    {
+        CV_ERROR(CV_StsNullPtr, "NULL name string");
+    }
+
+    window = cvGetWindow(name);
+    if (window == NULL)
+    {
+        CV_ERROR(CV_StsNullPtr, "NULL window");
+    }
+
+    result = (window.level == NSStatusWindowLevel) ? 1 : 0;
+
+    __END__;
+    return result;
+}
+
+void cvSetPropTopmost_COCOA( const char* name, const bool topmost )
+{
+    CVWindow *window = nil;
+    NSAutoreleasePool* localpool = nil;
+    CV_FUNCNAME( "cvSetPropTopmost_COCOA" );
+
+    __BEGIN__;
+    if( name == NULL )
+    {
+        CV_ERROR( CV_StsNullPtr, "NULL name string" );
+    }
+
+    window = cvGetWindow(name);
+    if ( window == NULL )
+    {
+        CV_ERROR( CV_StsNullPtr, "NULL window" );
+    }
+
+    if ([[window contentView] isInFullScreenMode])
+    {
+        EXIT;
+    }
+
+    localpool = [[NSAutoreleasePool alloc] init];
+    if (topmost)
+    {
+        [window makeKeyAndOrderFront:window.self];
+        [window setLevel:CGWindowLevelForKey(kCGMaximumWindowLevelKey)];
+    }
+    else
+    {
+        [window makeKeyAndOrderFront:nil];
+    }
+    [localpool drain];
+    __END__;
+}
+
 void cv::setWindowTitle(const String& winname, const String& title)
 {
     CVWindow *window = cvGetWindow(winname.c_str());
